#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab

TagNames = [
    "A",
    "ABBR",
    "ACRONYM",
    "ADDRESS",
    "ALTGLYPH",
    "ALTGLYPHDEF",
    "ALTGLYPHITEM",
    "ANIMATE",
    "ANIMATECOLOR",
    "ANIMATEMOTION",
    "ANIMATETRANSFORM",
    "ANNOTATION_XML",
    "APPLET",
    "AREA",
    "ARTICLE",
    "ASIDE",
    "AUDIO",
    "B",
    "BASE",
    "BASEFONT",
    "BDI",
    "BDO",
    "BGSOUND",
    "BIG",
    "BLINK",
    "BLOCKQUOTE",
    "BODY",
    "BR",
    "BUTTON",
    "CANVAS",
    "CAPTION",
    "CENTER",
    "CIRCLE",
    "CITE",
    "CLIPPATH",
    "CODE",
    "COL",
    "COLGROUP",
    "COLOR_PROFILE",
    "CURSOR",
    "DATA",
    "DATALIST",
    "DD",
    "DEFS",
    "DEL",
    "DESC",
    "DETAILS",
    "DFN",
    "DIALOG",
    "DIR",
    "DIV",
    "DL",
    "DT",
    "ELLIPSE",
    "EM",
    "EMBED",
    "FEBLEND",
    "FECOLORMATRIX",
    "FECOMPONENTTRANSFER",
    "FECOMPOSITE",
    "FECONVOLVEMATRIX",
    "FEDIFFUSELIGHTING",
    "FEDISPLACEMENTMAP",
    "FEDISTANTLIGHT",
    "FEFLOOD",
    "FEFUNCA",
    "FEFUNCB",
    "FEFUNCG",
    "FEFUNCR",
    "FEGAUSSIANBLUR",
    "FEIMAGE",
    "FEMERGE",
    "FEMERGENODE",
    "FEMORPHOLOGY",
    "FEOFFSET",
    "FEPOINTLIGHT",
    "FESPECTACTUALRLIGHTING",
    "FESPOTLIGHT",
    "FETILE",
    "FETURBULENCE",
    "FIELDSET",
    "FIGCAPTION",
    "FIGURE",
    "FILTER",
    "FONT",
    "FONT_FACE",
    "FONT_FACE_FORMAT",
    "FONT_FACE_NAME",
    "FONT_FACE_SRC",
    "FONT_FACE_URI",
    "FOOTER",
    "FOREIGNOBJECT",
    "FORM",
    "FRAME",
    "FRAMESET",
    "G",
    "GLYPH",
    "GLYPHREF",
    "H1",
    "H2",
    "H3",
    "H4",
    "H5",
    "H6",
    "HEAD",
    "HEADER",
    "HGROUP",
    "HKERN",
    "HR",
    "HTML",
    "I",
    "IFRAME",
    "IMAGE",
    "IMG",
    "INPUT",
    "INS",
    "ISINDEX",
    "KBD",
    "KEYGEN",
    "LABEL",
    "LEGEND",
    "LI",
    "LINE",
    "LINEARGRADIENT",
    "LINK",
    "LISTING",
    "MACTION",
    "MAIN",
    "MALIGNGROUP",
    "MALIGNMARK",
    "MAP",
    "MARK",
    "MARKER",
    "MARQUEE",
    "MASK",
    "MATH",
    "MENCLOSE",
    "MENU",
    "MENUITEM",
    "MERROR",
    "META",
    "METADATA",
    "METER",
    "MFENCED",
    "MFRAC",
    "MGLYPH",
    "MI",
    "MISSING_GLYPH",
    "MLABELEDTR",
    "MLONGDIV",
    "MMULTISCRIPTS",
    "MN",
    "MO",
    "MOVER",
    "MPADDED",
    "MPATH",
    "MPHANTOM",
    "MPRESCRIPTS",
    "MROOT",
    "MROW",
    "MS",
    "MSCARRIES",
    "MSCARRY",
    "MSGROUP",
    "MSLINE",
    "MSPACE",
    "MSQRT",
    "MSROW",
    "MSTACK",
    "MSTYLE",
    "MSUB",
    "MSUBSUP",
    "MSUP",
    "MTABLE",
    "MTD",
    "MTEXT",
    "MTR",
    "MULTICOL",
    "MUNDER",
    "MUNDEROVER",
    "NAV",
    "NEXTID",
    "NOBR",
    "NOEMBED",
    "NOFRAMES",
    "NONE",
    "NOSCRIPT",
    "OBJECT",
    "OL",
    "OPTGROUP",
    "OPTION",
    "OUTPUT",
    "P",
    "PARAM",
    "PATH",
    "PATTERN",
    "PLAINTEXT",
    "POLYGON",
    "POLYLINE",
    "PRE",
    "PROGRESS",
    "Q",
    "RADIALGRADIENT",
    "RB",
    "RECT",
    "RP",
    "RT",
    "RTC",
    "RUBY",
    "S",
    "SAMP",
    "SCRIPT",
    "SECTION",
    "SELECT",
    "SEMANTICS",
    "SET",
    "SMALL",
    "SOURCE",
    "SPACER",
    "SPAN",
    "STOP",
    "STRIKE",
    "STRONG",
    "STYLE",
    "SUB",
    "SUMMARY",
    "SUP",
    "SVG",
    "SWITCH",
    "SYMBOL",
    "TABLE",
    "TBODY",
    "TD",
    "TEMPLATE",
    "TEXT",
    "TEXTPATH",
    "TEXTAREA",
    "TFOOT",
    "TH",
    "THEAD",
    "TIME",
    "TITLE",
    "TR",
    "TRACK",
    "TREF",
    "TSPAN",
    "TT",
    "U",
    "UL",
    "USE",
    "VAR",
    "VIDEO",
    "VIEW",
    "VKERN",
    "WBR",
    "XMP",
]
